@@ -30,7 +30,28 @@
 // Linterna Mágica constructor
 function LinternaMagica(params)
 {
-    this.debug_level = params.debug;
+    // FIXME: This might be used in a frame and after the object is
+    // replaced, everithing else to be removed leaving only the
+    // video. This way it might be possible to play videos from remote
+    // sites in Epiphany (no GM_ API and xmlHttpReqeust is restricted to
+    // the same origin).
+        
+    if (window.top != window.self)
+    {
+	this.log("LinternaMagica.constructor:\n"+
+		 "Skipping (i)frame with address: "+
+		 window.location,1);
+	return null;
+    }
+
+    // The code above should be executed before the web logger
+    // element. Otherwise it is created and visible in iframes and
+    // objects that are used to embed clips in remote sites.
+
+    this.set_locale(params.locale);
+
+    // Could be a string, but we need integer.
+    this.debug_level = parseInt(params.debug);
 
     if (this.debug_level && params.log_to == "web")
     {
@@ -60,29 +81,12 @@ function LinternaMagica(params)
 	this.log_to = "console";
     }
 
-    // FIXME: This might be used in a frame and after the object is
-    // replaced, everithing else to be removed leaving only the
-    // video. This way it might be possible to play videos from remote
-    // sites in Epiphany (no GM_ API and xmlHttpReqeust is restricted to
-    // the same origin).
-
-    if (window.top != window.self)
+    var self = this;
+    var val = this.call_site_function_at_position.apply(self,[
+	"before_options_init",
+	window.location.hostname]);
+    if (!val)
     {
-	this.log("LinternaMagica.constructor:\n"+
-		 "Skipping (i)frame with address: "+
-		 window.location,1);
-	return null;
-    }
-
-    // Skip ted.com at the front page. With Gnash installed the flash
-    // object is created. The flashvars attrubute value is 24 KB
-    // (kilo*bytes*) and Firefox and forks block 
-    if (/ted\.com/i.test(window.location.hostname) &&
-	!/[A-Za-z0-9]+/i.test(window.location.pathname))
-    {	
-	this.log("LinternaMagica.constructor:\n"+
-		 "Skipping TED front page!"+
-		 " Blocks Firefox and forks.");
 	return null;
     }
 
@@ -92,8 +96,9 @@ function LinternaMagica(params)
     this.set_autostart(params.autostart);
     this.set_controls(params.controls);
     this.set_cookies(params.cookies);
-    this.set_wait_dailymotion(params.wait_dm);
+    this.set_wait_xhr(params.wait_xhr);
     this.set_check_updates(params.updates);
+    this.set_hd_link_quality(params.quality);
 
     // check_for_updates() MUST be called only if there is video object
     // found. The only place where the user can be informed is in the
@@ -104,11 +109,9 @@ function LinternaMagica(params)
     // Add the style sheet to the head of the document.
     this.create_stylesheet();
 
-    // Array of flash objects
-    this.dirty_objects = new Array();
-
-    // Array of created "video" objects
-    this.video_objects = new Array();
+    // Video flash objects counter. Counting starts from zero for the
+    // first found object with the first increment.
+    this.found_flash_video_objects = -1;
 
     // Object holding data for curently processed video ids with
     // XMLHttpRequest. Keys video_id+host , values 1/0.
@@ -121,21 +124,24 @@ function LinternaMagica(params)
 	this.player_timers = new Array();
     }
 
-    if (!this.plugin_is_installed &&
-	/dailymotion\.com/i.test(window.location.hostname))
+    var position_function = null ;
+    if (this.plugin_is_installed)
     {
-	this.request_video_link({video_id: window.location.pathname});
+	position_function = "flash_plugin_installed";
     }
-    // If there is a plugin installed do not search in scripts.
-    else if (!this.plugin_is_installed ||
-	     /myvideo\.de/i.test(window.location.hostname) ||
-	     /theonion\.com/i.test(window.location.hostname))
+    else
     {
-	this.log("LinternaMagica.constructor:\n"+
-		 "Examining scripts.", 4);
+	position_function = "no_flash_plugin_installed";
+    }
 
-	// video.google.* bloats in this function. It takes around 1 min
-	this.extract_objects_from_scripts();
+    var self = this;
+    var val = this.call_site_function_at_position.apply(self,[
+	position_function,
+	window.location.hostname]);
+    
+    if (!val)
+    {
+	return null;
     }
 
     this.log("LinternaMagica.constructor:\n"+
@@ -181,7 +187,7 @@ LinternaMagica.prototype.name =  "Linterna Mágica";
 
 // Release date string in POISIX time format (date +"%s")
 // FIXME: Add real string
-LinternaMagica.prototype.release_date = "1308131989";
+LinternaMagica.prototype.release_date = "1315406032";
 
 // The URL with information about the latest version. Must
 // return JSONP data:
@@ -193,27 +199,47 @@ LinternaMagica.prototype.updates_page =
     "http://linterna-magica.nongnu.org/downloads/updates.js";
 //    "http://localhost/linterna-magica/downloads/updates.js";
 
-LinternaMagica.prototype.description =
-    _("Watch video on the web ")+
-    _("in a brand new way: ")+
-    _("You don't need a glint, ")+
-    _("the magic lantern is ignited!");
+// When this is a function the internal _() function could be used.
+LinternaMagica.prototype.description = function()
+{
+    return (
+	this._("Watch video on the web ")+
+	    this._("in a brand new way: ")+
+	    this._("You don't need a glint, ")+
+	    this._("the magic lantern is ignited!")
+    );
+}
 
-LinternaMagica.prototype.license =
-    _("This program is free software; ")+
-    _("you can redistribute it and/or ")+
-    _("modify it under the terms of the ")+
-    _("GNU  General Public License (GNU GPL)")+
-    _(" version 3 (or later). ")+
-    _("A copy of the license can be downloaded from ");
+// When this is a function the internal _() function could be used.
+LinternaMagica.prototype.license = function()
+{
+    return (
+	this._("This program is free software; ")+
+	    this._("you can redistribute it and/or ")+
+	    this._("modify it under the terms of the ")+
+	    this._("GNU  General Public License (GNU GPL)")+
+	    this._(" version 3 (or later). ")+
+	    this._("A copy of the license can be downloaded from ")
+    );
+}
 
 LinternaMagica.prototype.license_link =
-    "http://www.gnu.org/licenses/gpl.html";
+    "https://www.gnu.org/licenses/gpl.html";
 
 LinternaMagica.prototype.homepage = 
     "http://linterna-magica.nongnu.org";
 // "http://localhost/lm";
 
+LinternaMagica.prototype.savannah_page =
+    "https://savannah.nongnu.org/projects/linterna-magica";
+
+LinternaMagica.prototype.bug_report_link = 
+    "https://sv.nongnu.org/support/?func=additem&amp;group=linterna-magica";
+
+LinternaMagica.prototype.microblog_link =
+    "https://identi.ca/group/linternamagica";
+
+
 // This is filled during build from the Makefile
 LinternaMagica.prototype.copyrights = new Array();
 
@@ -229,81 +255,91 @@ LinternaMagica.prototype.player.init = function(id)
     this.player.set_player_name.apply(this,[id]);
     var self = this;
 
-    // We only start the firs clip
-    if (this.autostart && id ==0)
+    var started_clip = this.find_started_clip();
+
+    // Only start the clip, if not other is playing.
+    if (this.autostart && started_clip == null)
     {
 	// Sometimes it skips seconds if the
 	// interval is 1sec
-	this.player_timers[id] = setInterval(function()
-					     {
-						 self.ticker.apply(self,[id]);
-					     }, 500);
+	this.player_timers[id] = 
+	    setInterval(function()
+			{
+			    self.ticker.apply(self,[id]);
+			}, 500);
     }
 
+    var volume_interval_function =    function()
+    {
+	var knob =
+	    document.getElementById("linterna-magica-controls-"+
+				    "volume-slider-knob-"+id);
+
+	if (!knob)
+	{
+	    return null;
+	}
+
+	var text = knob.nextSibling;
+	var slider = knob.parentNode;
+	var vol = null;
+
+	var video_object = self.get_video_object(id);
+
+	var player_name = video_object.getAttribute("player_name");
+
+	if (/gecko/i.test(player_name)
+	    || /quicktime plug-in/i.test(player_name))
+	{
+	    try
+	    {
+		vol = video_object.GetVolume();
+
+		if (/quicktime/i.test(player_name))
+		{
+		    // totemNarrowspace uses 255 as max
+		    // calculate as 100 %
+		    vol = parseInt(vol * 100/255);
+		}
+	    }
+	    catch(e)
+	    {
+	    }
+	}
+	else if (/vlc/i.test(player_name))
+	{
+	    if (video_object.audio)
+	    {
+		vol =
+		    video_object.audio.
+		    volume;
+	    }
+	}
+
+	if (vol)
+	{
+	    var pos = 
+		parseInt((slider.clientWidth*vol/100) -
+			 knob.clientWidth-knob.clientWidth/2);
+
+	    knob.style.setProperty("left", pos+"px", "important");
+
+	    text.textContent = vol+"%";
+
+	    clearInterval(volume_interval);
+	}
+    }
 
     var volume_interval =
-	setInterval(function()
-		    {
-			var knob = document.getElementById(
-			    "linterna-magica-controls-"+
-				"volume-slider-knob-"+id);
-
-			if (!knob)
-			    return;
-
-			var text = knob.nextSibling;
-			var slider = knob.parentNode;
-			var vol = null;
-
-			var video_object = self.
-			    get_video_object(id);
-
-			var player_name =
-			    video_object.
-			    getAttribute("player_name");
-
-			if (/gecko/i.test(player_name)
-			    || /quicktime plug-in/i.test(player_name))
-			{
-			    try
-			    {
-				vol =
-				    video_object.GetVolume();
-				if (/quicktime/i.test(player_name))
-				{
-				    // totemNarrowspace uses 255 as max
-				    // calculate as 100 %
-				    vol = parseInt(vol * 100/255);
-				}
-
-			    }
-			    catch(e)
-			    {
-			    }
-			}
-			else if (/vlc/i.test(player_name))
-			{
-			    if (video_object.audio)
-			    {
-				vol =
-				    video_object.audio.
-				    volume;
-			    }
-			}
-
-			if (vol)
-			{
-			    var pos = parseInt(
-				(slider.clientWidth*vol/100) -
-				    knob.clientWidth-knob.clientWidth/2);
-
-			    knob.style.setProperty("left", pos+"px",
-						   "important");
-
-			    text.textContent = vol+"%";
-
-			    clearInterval(volume_interval);
-			}
-
-		    }, 800);
+	setInterval(volume_interval_function, 800);
 }
+
+// Localization languages object
+LinternaMagica.prototype.languages = new Object();
+
+// No translation.
+LinternaMagica.prototype.languages["C"] = 
+    {
+	__direction: "ltr",
+	__translators: null,
+    };
