@@ -39,8 +39,6 @@ LinternaMagica.prototype.create_video_object = function(object_data)
 	     "Creating video object with linterna_magica_id "+id,2);
 
     // Check if another object is created already there and skip.
-    // this.dirty_objects[(id-1)] will brak for scripts, because it is
-    // null
     if (((id-1) >= 0))
     {
 	// Will break if the there is object that is not created just
@@ -69,10 +67,25 @@ LinternaMagica.prototype.create_video_object = function(object_data)
 
     container.setAttribute("id", "linterna-magica-"+id);
     container.setAttribute("class", "linterna-magica");
+
+    // Fix for rtl languages. See
+    // https://savannah.nongnu.org/bugs/?32740
+    // https://savannah.nongnu.org/task/?11056
+    container.setAttribute("dir", this.languages[this.lang].__direction);
+
+    var lang_code = this.lang.split("_")[0];
+    if (!lang_code)
+    {
+	// The locale is C
+	lang_code = "en";
+    }
+
+    container.setAttribute("xml:lang", lang_code);
+    container.setAttribute("lang", lang_code);
+
     container.style.setProperty("width",
 				(object_data.width+"px"), "important");
 
-
     header.setAttribute("class", "linterna-magica-header");
     header.setAttribute("id", "linterna-magica-header-"+id);
     header.style.setProperty("width",
@@ -82,26 +95,36 @@ LinternaMagica.prototype.create_video_object = function(object_data)
 
     script_name.textContent = "Linterna Mágica";
     script_name.setAttribute("href", "#");
-    script_name.setAttribute("title", _("About")+ " Linterna Mágica");
+    script_name.setAttribute("title", this._("About")+ " Linterna Mágica");
     script_name.setAttribute("id", "linterna-magica-logo-"+id);
     script_name.setAttribute("class", "linterna-magica-logo");
 
     var self = this;
-    script_name.addEventListener("click", function(ev){
-	var el = this;
-	self.about.apply(self, [ev, el]);
-    }, false);
+    script_name.addEventListener("click", function(ev)
+				 {
+				     var el = this;
+				     self.about.apply(self, [ev, el]);
+				 }, false);
+
+    var site_html5_player =
+	this.find_site_html5_player_wrapper(object_data.parent);
+
+    var toggle_plugin_switch_type = 
+	site_html5_player ? "html5" : "plugin";
 
     // If the plugin is not installed this is useless
-    if (this.plugin_is_installed)
+    if (this.plugin_is_installed || site_html5_player)
     {
 	// append before download link (first button in the header)
-	var toggle_plugin = this.create_toggle_plugin_link(null,id);
+	var toggle_plugin = 
+	    this.create_toggle_plugin_link(null,id,
+					   toggle_plugin_switch_type);
+
 	header.appendChild(toggle_plugin);
     }
 
-    dw_link.textContent = _("Download");
-    dw_link.setAttribute("title", _("Save the video clip"));
+    dw_link.textContent = this._("Download");
+    dw_link.setAttribute("title", this._("Save the video clip"));
 
     dw_link.setAttribute("id", "linterna-magica-video-download-link-"+id);
     dw_link.setAttribute("class", "linterna-magica-video-download-link");
@@ -117,18 +140,31 @@ LinternaMagica.prototype.create_video_object = function(object_data)
     // Create HD links
     if (object_data.hd_links)
     {
+	var preferred_link = 
+	    this.compute_preferred_hd_link(object_data.hd_links);
+
+	// No link is calculated. Set to lowest.
+	if (preferred_link == null || isNaN(preferred_link))
+	{
+	    preferred_link = 
+		object_data.hd_links[object_data.hd_links.length-1];
+	}
+
 	var hd_wrapper = document.createElement("div");
 	var hd_button = document.createElement("a");
 	hd_button.setAttribute("href","#");
-	hd_button.textContent = _("HQ");
-	hd_button.setAttribute("title", _("Higher quality"));
+	hd_button.textContent = this._("HQ");
+	hd_button.setAttribute("title", this._("Higher quality"));
 	hd_button.setAttribute("class", "linterna-magica-switch-hd");
-	hd_button.addEventListener("click", function(ev)
-				   {
-				       var el = this;
-				       self.show_or_hide_hd_links.
-					   apply(self, [ev, el]);
-				   }, false);
+
+	var hd_button_click_function =  function(ev)
+	{
+	    var el = this;
+	    self.show_or_hide_hd_links.apply(self, [ev, el]);
+	};
+
+	hd_button.addEventListener("click",
+				   hd_button_click_function, false);
 
 	hd_wrapper.appendChild(hd_button);
 
@@ -145,12 +181,27 @@ LinternaMagica.prototype.create_video_object = function(object_data)
 	    var button = document.createElement("a");
 	    button.setAttribute("href",object_data.hd_links[link].url);
 	    button.textContent = object_data.hd_links[link].label;
-	    button.addEventListener("click", function(ev)
-				    {
-					var el = this;
-					self.switch_to_hd_link.
-					    apply(self, [ev, el]);
-				    }, false);
+
+	    var button_click_function = function(ev)
+	    {
+		var el = this;
+		self.switch_to_hd_link.apply(self, [ev, el]);
+	    };
+
+	    button.addEventListener("click",
+				    button_click_function , false);
+
+	    // Preferred link 
+	    if (link == preferred_link)
+	    {
+		// Set the link in the interface
+		this.select_hd_link_in_list(button,id);
+
+		// Set the link for the player and download link.
+		object_data.link = object_data.hd_links[link].url;
+		dw_link.setAttribute("href",
+				     object_data.hd_links[link].url);
+	    }
 
 	    li.appendChild(button);
 	    ul.appendChild(li);
@@ -166,15 +217,18 @@ LinternaMagica.prototype.create_video_object = function(object_data)
     if (this.debug_level && this.log_to == "web")
     {
 	var log_link  =  this.create_web_log_link(id);
-	log_link.addEventListener("click", function(ev)
-				  {
-				      var el = this;
-				      self.show_or_hide_web_log.
-					  apply(self, [ev, el]);
-				  }, false);
+
+	var log_link_click_function = function(ev)
+	{
+	    var el = this;
+	    self.show_or_hide_web_log.apply(self, [ev, el]);
+	};
+
+	log_link.addEventListener("click",
+				  log_link_click_function, false);
 
 	header.appendChild(log_link);
-	// Hide the web log, so it accessible only from interface.
+	// Hide the web log, so it is accessible only from the interface.
 	var log = document.getElementById("linterna-magica-web-log");
 	log.style.setProperty("display","none", "important");
     }
@@ -182,7 +236,7 @@ LinternaMagica.prototype.create_video_object = function(object_data)
     object_tag.setAttribute("width", object_data.width);
     object_tag.setAttribute("height", object_data.height);
     object_tag.setAttribute("id","linterna-magica-video-object-"+id);
-    object_tag.setAttribute("standby", _("Loading video..."));
+    object_tag.setAttribute("standby", this._("Loading video..."));
 
     if (object_data.link)
     {
@@ -209,11 +263,14 @@ LinternaMagica.prototype.create_video_object = function(object_data)
 	object_tag.setAttribute("data", object_data.link);
     }
 
-    message.textContent = _("Waiting for video plugin...");
+    message.textContent = this._("Waiting for video plugin...");
 
     param.setAttribute("name", "autoplay");
-    // Start only the first found clip
-    param.setAttribute("value", (id > 0) ? "false" : this.autostart);
+    // Find if a clip is already playing.
+    var started_clip = this.find_started_clip();
+
+    param.setAttribute("value",
+		       (started_clip !== null) ? "false" : this.autostart);
 
     object_tag.appendChild(param);
 
@@ -253,25 +310,35 @@ LinternaMagica.prototype.create_video_object = function(object_data)
 				(parseInt(object_data.height)-20)+"px",
 				"important");
     container.appendChild(about_box);
-
-    object_tag.setAttribute("linterna_magica_id", id);
+    
+    // Mark the object, so it is not processed and chacked after
+    // insertion. The function mark_flash_object will use new id and
+    // that might not be a good idea (object count, XHRs, duplicate
+    // objects ... ). The linterna_magica_id property is set ot a
+    // float, so it is not the same as the one of the flash object.
+    object_tag.linterna_magica_id =
+	parseFloat(object_data.linterna_magica_id+".1");
 
     // Add link after the object/embed
     // this.set_priority() has set this.priority
     // to self if there is no plugin
-    if (this.plugin_is_installed)
+	if (this.plugin_is_installed || site_html5_player)
     {
-	toggle_plugin = this.create_toggle_plugin_link(this.priority, id);
+	toggle_plugin =
+	    this.create_toggle_plugin_link("link-not-in-header", id,
+					  toggle_plugin_switch_type);
 
-	// Fix displacement of toggle_plugin link/button in vimeo
-	if (/vimeo\.com/i.test(window.location.hostname))
+	var before = null;
+
+	if (this.plugin_is_installed && !site_html5_player)
 	{
-	    toggle_plugin.style.setProperty("top", 
-					    parseInt(object_data.height)+10+
-					    "px", "important");
+	    before = this.get_flash_video_object(id).nextSibling;
+	}
+	else if (site_html5_player)
+	{
+	    before = site_html5_player.nextSibling;
 	}
 
-	var before = this.dirty_objects[id].nextSibling;
 	if (before)
 	{
 	    object_data.parent.insertBefore(toggle_plugin, before);
@@ -280,8 +347,14 @@ LinternaMagica.prototype.create_video_object = function(object_data)
 	{
 	    object_data.parent.appendChild(toggle_plugin);
 	}
-	if (!/plugin/i.test(this.priority))
+
+	if (((this.priority.self > this.priority.plugin) && 
+	     this.plugin_is_installed && !site_html5_player) ||
+	    ((this.priority.self > this.priority.html5) &&
+	     site_html5_player))
 	{
+	    // Hide the toggle plugin button only if self has higher
+	    // priority then plugin and html5.
 	    toggle_plugin.style.setProperty("display", "none",
 					    "important");
 	}
@@ -293,30 +366,42 @@ LinternaMagica.prototype.create_video_object = function(object_data)
 	container.appendChild(controls);
     }
 
-    // IMPORTANT: First remove the object
-    // then insert the new one, otherwise a loop is created
-    // in extract_objects_from_dom.
+    var dom_object =  this.get_flash_video_object(id);
 
-    // Remove the object if it is in DOM
-    // Objects extracted from script has null value in this.dirty_objects
-    if (/self/i.test(this.priority)
-	&& this.dirty_objects[id]
-	// The object is still in DOM some scripts remove it
-       && this.dirty_objects[id].parentNode)
+    if (!dom_object)
     {
-	if(this.dirty_objects[id].nextSibling)
-	{
-	    object_data.use_sibling = this.dirty_objects[id].nextSibling;
-	}
-
-	object_data.parent.removeChild(this.dirty_objects[id]);
+	dom_object = site_html5_player;
     }
 
-    if (/self/i.test(this.priority) && this.plugin_is_installed)
+    // Remove/hide the object if it is in DOM
+    if ((((this.priority.self > this.priority.plugin) &&
+	  !site_html5_player) || 
+	 ((this.priority.self > this.priority.html5) &&
+	  site_html5_player)) &&
+	dom_object &&
+	// The object is still in DOM some scripts remove it
+	dom_object.parentNode)
+    {
+	if(dom_object.nextSibling)
+	{
+	    object_data.use_sibling = dom_object.nextSibling;
+	}
+
+	if (!site_html5_player)
+	{
+	    this.hide_flash_video_object(id,dom_object.parentNode);
+	}
+	else 
+	{
+	    this.hide_site_html5_player(object_data.parent);
+	}
+    }
+
+    if (toggle_plugin)
     {
 	object_data.parent.insertBefore(container, toggle_plugin);
     }
-    else if(/self/i.test(this.priority))
+    else
     {
 	if (object_data.use_sibling)
 	{
@@ -333,100 +418,55 @@ LinternaMagica.prototype.create_video_object = function(object_data)
 	}
     }
 
-    // Various CSS fix-ups
+    if (((this.priority.self < this.priority.plugin) && 
+	 this.plugin_is_installed) || 
+	((this.priority.self < this.priority.html5) && 
+	 site_html5_player))
+    {
+	this.hide_lm_video(object_data.linterna_magica_id);
+    }
+
+    // Defaults style fixes applied always.
+    about_box.style.setProperty("overflow", "auto", "important");
+
     container.style.setProperty("height",
 				((parseInt(object_data.height)+26+
 				  (this.controls ? 24 : 0))+"px"),
 				"important");
 
-    // In channels/user pages in YouTube the web controlls are
-    // overlapped by this element.
-    if (/youtube\.com/i.test(window.location.hostname) &&
-	document.getElementById("playnav-playview"))
-    {
-	var el = 	document.getElementById("playnav-playview");
-	el.style.setProperty("margin-top", "50px", "important");
-    }
-
-    // The thumbnail image overlaps the toggle plugin button after our
-    // changes. This way our button is visible.
-    if (/vimeo\.com/i.test(window.location.hostname) && 
-	object_data.parent.firstChild)
-    {
-	// The first child should be a div with thumbnail as
-	// background. Reduce it's size so it will not overlap our
-	// button.
-	object_data.parent.firstChild.style.
-	    setProperty("height", parseInt(object_data.height)+"px",
-			"important");
-    }
-
     if (object_data.parent)
     {
-	var move_down_fb_frame = null;
-	if (/mqsto\.com/i.test(window.location.hostname))
-	{
-	    // Move the facebook comment frame in mqsto.com 100px
-	    // down. overlaps the player.
-	    move_down_fb_frame = 100;
-	}
-
 	object_data.parent.style.setProperty("height",
 					     (parseInt(object_data.height)+
-					      26+(move_down_fb_frame ? 
-						  move_down_fb_frame: 0)+
+					      26+
 					      // borders 1px x 2
 					      2+
 					      (this.controls ? 24 : 0))+"px",
 					     "important");
-
-	// Show HD links list. 
-	if (/vimeo\.com/i.test(window.location.hostname))
-	{
-	    object_data.parent.style.
-	    	setProperty("overflow", "visible", "important");
-
-	    object_data.parent.parentNode.style.
-	    	setProperty("overflow", "visible", "important");
-	}
-	// if (/vimeo\.com/i.test(window.location.hostname))
-	// {
-	//     // object_data.parent.parentNode.style.
-	//     // 	setProperty("overflow-y", "visible", "important");
-
-	//     // Hides the HD links menu
-	//     // object_data.parent.parentNode.style.
-	//     // 	setProperty("overflow-x", "hidden", "important");
-	// }
     }
 
+    
+
+    var dom_object = this.get_flash_video_object(id);
     // Objects extracted from script usually does not have cloned object
     // For example youtube
-    if (this.dirty_objects[id])
+    if (dom_object)
     {
 	// Prevent the object to fill the container at 100% (if set)
 	// This way the toggle plugin link after the object is not
 	// overlaping elements.
-	this.dirty_objects[id].style.setProperty("height",
-						 object_data.height+"px",
-						 "important");
+	dom_object.style.setProperty("height", object_data.height+"px",
+				     "important");
     }
+
     // Prevent the video object to fill the area at 100% and overlap
     // other elements. This happends if the site CSS sets this
-    object_tag.style.setProperty("height",
-				 object_data.height+"px",
+    object_tag.style.setProperty("height", object_data.height+"px",
 				 "important");
 
-    object_tag.style.setProperty("width",
-				 object_data.width+"px",
+    object_tag.style.setProperty("width", object_data.width+"px",
 				 "important");
 
-    // No idea what this fixes.
-    if (/vimeo\.com/i.test(window.location.href))
-    {
-	object_tag.style.setProperty("position","relative","important");
-    }
-
     if (this.plugin_is_installed)
     {
 	// Remove backrounds so the toggle plugin link is visible
@@ -449,66 +489,6 @@ LinternaMagica.prototype.create_video_object = function(object_data)
     // parent.parentNode is not an object
     if (object_data.parent && object_data.parent.parentNode)
     {
-
-	// The CSS hides parts of our elements
-	if (/tv7\.bg/i.test(window.location.hostname) ||
-	    /vimeo\.com/i.test(window.location.hostname))
-	{
-	    object_data.parent.parentNode.style.
-		setProperty("height",
-			    (parseInt(object_data.height)+26+
-			     // borders 1px x 2
-			     2+
-			     (this.controls ? 24 : 0)  )+"px",
-			    "important");
-
-	    object_data.parent.parentNode.style.
-		setProperty("width",
-			    (parseInt(object_data.width+2))+"px",
-			    "important");
-	}
-
-	if (/tv7\.bg/i.test(window.location.hostname) ||
-	    /vimeo\.com/i.test(window.location.hostname))
-	{
-	    var third_parent = object_data.parent.parentNode.parentNode;
-	    if (third_parent)
-	    {
-		third_parent.style.setProperty("overflow", "visible", "important");
-		third_parent.style.
-		    setProperty("height", 
-				(parseInt(object_data.height)+26+
-				 // borders 1px x 2
-				 2+
-				 (this.controls ? 24 : 0)  )+"px",
-				"important");
-	    }
-	}
-
-	if (/vimeo\.com/i.test(window.location.hostname) ||
-	    /reuters\.com/i.test(window.location.hostname))
-	{
-	    // Extra height for reuters.com. Otherwise the controlls
-	    // are hidden.
-	    var reuters = 
-		/reuters\.com/i.test(window.location.hostname) ? 100 : 0;
-
-	    var fourth_parent = object_data.parent.parentNode.parentNode.parentNode;
-	    if (fourth_parent)
-	    {
-		fourth_parent.style.setProperty("overflow", "visible", "important");
-		fourth_parent.style.
-		    setProperty("height", 
-				(parseInt(object_data.height)+26+
-				 reuters+
-				 // borders 1px x 2
-				 2+
-				 (this.controls ? 24 : 0)  )+"px",
-				"important");
-	    }
-	}
-
-
 	// dailymotion.com && metacafe.com && btn.bg
 	// These have single div inside another one
 	// which (through height) overlaps elemenst
@@ -536,23 +516,10 @@ LinternaMagica.prototype.create_video_object = function(object_data)
 	}
     }
 
-    // Bug #33504 https://savannah.nongnu.org/bugs/?33504
-    if (/youtube\.com/i.test(window.location.hostname))
-    {
-	object_data.parent.style.setProperty("overflow", 
-					     "visible", "important");
-    }
-
-    // Temporary
-    // parent.style.setProperty("border", "1px solid red", "important");
-
-    // Push to video object list
-    this.video_objects.push(container);
-
     // Init the web controls functions
     // only if Linterna Mágica has priority
     if (this.controls &&
-	/self/i.test(this.priority))
+	(this.priority.self > this.priority.plugin))
     {
 	this.player.init.apply(this,[id]);
     }
@@ -560,5 +527,11 @@ LinternaMagica.prototype.create_video_object = function(object_data)
     // Examine the option for updates and check if necessary.
     this.check_for_updates();
 
+    // Various CSS fixes
+    var self = this;
+    var val = this.call_site_function_at_position.apply(self,[
+	"css_fixes",
+	window.location.hostname, object_data]);
+
     return null;
 }
