@@ -28,17 +28,7 @@
 // END OF LICENSE HEADER
 
 
-// Find the current video object in the DOM
-LinternaMagica.prototype.get_video_object = function(id)
-{
-    var video_object  = window.document.
-	getElementById("linterna-magica-video-object-"+id);
-
-    return video_object;
-}
-
-
-// Set player_name attribute to video objects.
+// Set lm_player_name property
 // This is used to know wchich API to use
 // navigator.plugins[x].name:
 // Tototem:  VLC Multimedia Plugin (compatible Totem 2.30.1)
@@ -48,10 +38,17 @@ LinternaMagica.prototype.get_video_object = function(id)
 // NOTE:
 // navigator.mimeTypes["video/flv"].enabledPlugin.name is not
 // reliable if more than one plugin is installed.
+// This is done once, and all the other functions just check a string.
 LinternaMagica.prototype.player.set_player_name = function(id)
 {
     var name = null;
     var video_object = this.get_video_object(id);
+
+    if (!video_object)
+    {
+	return null;
+    }
+
     var mimeTypes = navigator.mimeTypes;
 
     var mime = mimeTypes[video_object.getAttribute("type")];
@@ -69,7 +66,21 @@ LinternaMagica.prototype.player.set_player_name = function(id)
     {
 	this.log("LinternaMagica.player.set_player_name:\n"+
 		 "Name set to "+name,3);
-	video_object.setAttribute("player_name", name);
+	video_object.lm_player_name = name;
+    }
+
+    return name;
+}
+
+// Return the name ofthe plugin that will play the video.
+LinternaMagica.prototype.get_player_name = function(id)
+{
+    var name = null;
+    var video_object = this.get_video_object(id);
+
+    if (video_object)
+    {
+	name = video_object.lm_player_name;
     }
 
     return name;
@@ -90,11 +101,12 @@ LinternaMagica.prototype.player.set_player_name = function(id)
 LinternaMagica.prototype.player.state = function(id)
 {
     var video_object = this.get_video_object(id);
+    var player_name = this.get_player_name(id);
 
-    if (!video_object)
+    if (!video_object || !player_name)
+    {
 	return null;
-
-    var player_name = video_object.getAttribute("player_name");
+    }
 
     var time = new Object();
     time.duration = null;
@@ -106,10 +118,10 @@ LinternaMagica.prototype.player.state = function(id)
 	switch(video_object.playState)
 	{
 	case 0:
-	    time.state = _("Loading");
+	    time.state = this._("Loading");
 	    break;
 	case  6:
-	    time.state = _("Buffering");
+	    time.state = this._("Buffering");
 	    break;
 
 	}
@@ -143,17 +155,17 @@ LinternaMagica.prototype.player.state = function(id)
 	    switch (video_object.input.state)
 	    {
 	    case 0:
-		time.state = _("Loading");
+		time.state = this._("Loading");
 		break;
 	    case 2:
-		time.state = _("Buffering");
+		time.state = this._("Buffering");
 		break;
 	    }
 	}
 
 	if (!time.state && video_object.input)
 	{
-	    // Fix exception when switching between
+	    // Fixes exception when switching between
 	    // Linterna Mágica and flash plugin
 	    try
 	    {
@@ -180,7 +192,7 @@ LinternaMagica.prototype.player.state = function(id)
 	    // Xine supports only playing (3) and paused (4)
 	    var state = video_object.controls.GetPlayState();
 	    if (state !== 4 && state !== 3)
-		time.state = _("Loading");
+		time.state = this._("Loading");
 	}
 	catch(e)
 	{
@@ -189,7 +201,7 @@ LinternaMagica.prototype.player.state = function(id)
 
 	if (!time.state)
 	{
-	    // Fix exception when switching between
+	    // Fixes exception when switching between
 	    // Linterna Mágica and flash plugin
 	    try
 	    {
@@ -197,7 +209,7 @@ LinternaMagica.prototype.player.state = function(id)
 		time.duration = video_object.controls.GetLength()/1000;
 		// Xine does not have percent
 		time.percent =  (time.position/time.duration);
- 		// Fix NaN result (division by 0)
+ 		// Fixes NaN result (division by 0)
 		time.percent = time.percent ? time.percent : 0;
 	    }
 	    catch(e)
@@ -222,16 +234,16 @@ LinternaMagica.prototype.player.state = function(id)
 	{
 	case "Loading":
 	case "Waiting":
-	    time.state = _("Loading");
+	    time.state = this._("Loading");
 	    break;
 	case "NOT_READY":
-	    time.state = _("Waiting plugin");
+	    time.state = this._("Waiting plugin");
 	    break;
 	}
 
 	if (!time.state)
 	{
-	    // Fix exception when switching between
+	    // Fixes exception when switching between
 	    // Linterna Mágica and flash plugin
 	    try
 	    {
@@ -241,7 +253,7 @@ LinternaMagica.prototype.player.state = function(id)
 		// return the same format as getPercent() in
 		// gecko-mediaplayer
 		time.percent = (time.position/time.duration);
- 		// Fix NaN result (division by 0)
+ 		// Fixes NaN result (division by 0)
 		time.percent = time.percent ? time.percent : 0;
 	    }
 	    catch(e)
@@ -260,7 +272,6 @@ LinternaMagica.prototype.player.state = function(id)
 	}
     }
 
-    // new
     var sec_pos = Math.round(time.position) % 60;
     var min_pos = Math.round(time.position / 60) % 60;
     var hour_pos = Math.round(time.position / 3600);
@@ -284,7 +295,12 @@ LinternaMagica.prototype.player.state = function(id)
 LinternaMagica.prototype.player.pause = function(id)
 {
     var video_object = this.get_video_object(id);
-    var player_name = video_object.getAttribute("player_name");
+    var player_name = this.get_player_name(id);
+
+    if (!video_object || !player_name)
+    {
+	return null;
+    }
 
     if (/gecko-mediaplayer/.test(player_name))
     {
@@ -310,10 +326,15 @@ LinternaMagica.prototype.player.pause = function(id)
 LinternaMagica.prototype.player.play = function(id)
 {
     var video_object = this.get_video_object(id);
-    var player_name = video_object.getAttribute("player_name");
+    var player_name = this.get_player_name(id);
 
-    if (/gecko-mediaplayer/.test(player_name)
-	|| /totem/.test(player_name))
+    if (!video_object || !player_name)
+    {
+	return null;
+    }
+
+    if (/gecko-mediaplayer/.test(player_name) ||
+	/totem/.test(player_name))
     {
 	video_object.Play();
     }
@@ -336,7 +357,12 @@ LinternaMagica.prototype.player.play = function(id)
 LinternaMagica.prototype.player.stop = function(id)
 {
     var video_object = this.get_video_object(id);
-    var player_name = video_object.getAttribute("player_name");
+    var player_name = this.get_player_name(id);
+
+    if(!video_object || !player_name)
+    {
+	return null;
+    }
 
     if (/gecko-mediaplayer/.test(player_name))
     {
@@ -362,15 +388,21 @@ LinternaMagica.prototype.player.stop = function(id)
 LinternaMagica.prototype.player.forward = function(id,time)
 {
     var video_object = this.get_video_object(id);
-    var player_name = video_object.getAttribute("player_name");
+    var player_name = this.get_player_name(id);
+
+    if (!video_object || !player_name)
+    {
+	return null;
+    }
+
     // 10 seconds
     if (!time)
     {
 	time = 10000;
     }
 
-    if (/gecko-mediaplayer/.test(player_name)
-	|| /quicktime plug-in/i.test(player_name))
+    if (/gecko-mediaplayer/.test(player_name) ||
+	/quicktime plug-in/i.test(player_name))
     {
 	if (/%/.test(time))
 	{
@@ -432,7 +464,12 @@ LinternaMagica.prototype.player.forward = function(id,time)
 LinternaMagica.prototype.player.rewind = function(id,time)
 {
     var video_object = this.get_video_object(id);
-    var player_name = video_object.getAttribute("player_name");
+    var player_name = this.get_player_name(id);
+
+    if (!video_object || !player_name)
+    {
+	return null;
+    }
 
     if (!time)
 	time = 10000;
@@ -498,7 +535,12 @@ LinternaMagica.prototype.player.rewind = function(id,time)
 LinternaMagica.prototype.player.fullscreen = function(id)
 {
     var video_object = this.get_video_object(id);
-    var player_name = video_object.getAttribute("player_name");
+    var player_name = this.get_player_name(id);
+
+    if (!video_object || !player_name)
+    {
+	return null;
+    }
 
     if (/gecko-mediaplayer/.test(player_name))
     {
@@ -527,7 +569,12 @@ LinternaMagica.prototype.player.fullscreen = function(id)
 LinternaMagica.prototype.player.set_volume = function (id, volume)
 {
     var video_object = this.get_video_object(id);
-    var player_name = video_object.getAttribute("player_name");
+    var player_name = this.get_player_name(id);
+
+    if (!video_object || !player_name)
+    {
+	return null;
+    }
 
     // The slider control function returns values as string with % in
     // them.
@@ -558,30 +605,35 @@ LinternaMagica.prototype.player.set_volume = function (id, volume)
 LinternaMagica.prototype.player.toggle_mute = function (id)
 {
     var video_object = this.get_video_object(id);
-    var player_name = video_object.getAttribute("player_name");
+    var player_name = this.get_player_name(id);
+
+    if (!video_object || !player_name)
+    {
+	return null;
+    }
 
     var vol = null;
 
-    if (/gecko-mediaplayer/.test(player_name)
+    if (/gecko-mediaplayer/.test(player_name) ||
 	// totemNarrowspace plugin (quicktime)
-	|| /quicktime plug-in/i.test(player_name))
+	/quicktime plug-in/i.test(player_name))
     {
 	// totemNarrowspace has get/set mute but
 	// can not make it work
 	// gecko do not have mute method in the API
 	// Mute
-	if (!video_object.hasAttribute("linterna_magica_volume"))
+	if (!video_object.lm_player_volume)
 	{
 	    vol = video_object.GetVolume();
-	    video_object.setAttribute("linterna_magica_volume", vol);
+	    video_object.lm_player_volume, vol;
 	    video_object.SetVolume(0);
 	}
 	// unMute
 	else
 	{
-	    vol = video_object.getAttribute("linterna_magica_volume");
+	    vol = video_object.lm_player_volume;
 	    video_object.SetVolume(parseInt(vol));
-	    video_object.removeAttribute("linterna_magica_volume");
+	    delete video_object.lm_player_volume;
 	    // totemNarrowspace uses 255 as max value
 	    // calculcate as 100
 	    if (/quicktime/i.test(player_name))
@@ -639,9 +691,22 @@ LinternaMagica.prototype.slider_control = function(event)
     }
 
     if (!knob)
+    {
 	return null;
+    }
 
-    var old_position = parseInt(knob.style.getPropertyValue("left"));
+    var move = null;
+    if (this.languages[this.lang].__direction == "ltr" ||
+	this.languages[this.lang].__direction !== "rtl")
+    {
+	move = "left";
+    }
+    else if (this.languages[this.lang].__direction == "rtl")
+    {
+	move = "right";
+    }
+
+    var old_position = parseInt(knob.style.getPropertyValue(move));
     var direction = 0;
     var position = old_position;
 
@@ -657,8 +722,6 @@ LinternaMagica.prototype.slider_control = function(event)
     }
 
 
-
-
     // Get wheel direction
     if (event.type == "mousewheel")
     {
@@ -694,6 +757,13 @@ LinternaMagica.prototype.slider_control = function(event)
 		offset_left-
 		knob.clientWidth/2;
 
+	    // When the direction is rtl the knob starts at
+	    // slider.clientWidth, and moves to 0.
+	    if (move == "right")
+	    {
+		position = Math.abs(position- slider.clientWidth);
+	    }
+
 	    if (position > old_position)
 	    {
 		direction = 1;
@@ -744,12 +814,12 @@ LinternaMagica.prototype.slider_control = function(event)
 	position = 0;
     }
 
-    knob.style.setProperty("left",
+    knob.style.setProperty(move,
 			   position +"px",
 			   "important");
 
     var return_data = new Object();
-    return_data.val =  (parseInt((position/slider.clientWidth) *100))+"%";
+    return_data.val = (parseInt((position/slider.clientWidth) *100))+"%";
     return_data.direction = direction;
     return return_data;
 }
@@ -770,13 +840,15 @@ LinternaMagica.prototype.ticker = function(id)
     // With it the logic kills the ticker before  the plugin starts
 
     if (!time_and_state)
+    {
 	return;
+    }
 
     // Prevent clearing before the clip starts
     // Clear when the video end is reached
     // or null returned (no video_object found)
-    if ((time_and_state.position && time_and_state.duration)
-	&& (time_and_state.position >= time_and_state.duration))
+    if ((time_and_state.position && time_and_state.duration) &&
+	(time_and_state.position >= time_and_state.duration))
     {
 	clearInterval(self.player_timers[id]);
 	delete self.player_timers[id];
@@ -802,6 +874,18 @@ LinternaMagica.prototype.ticker = function(id)
 
     if (knob)
     {
+	var move = null;
+
+	if (this.languages[this.lang].__direction == "ltr" ||
+	    this.languages[this.lang].__direction !== "rtl")
+	{
+	    move = "left";
+	}
+	else if (this.languages[this.lang].__direction == "rtl")
+	{
+	    move = "right";
+	}
+
 	var slider = knob.parentNode;
 	var pos = parseInt(slider.clientWidth *
 			   time_and_state.percent);
@@ -809,7 +893,7 @@ LinternaMagica.prototype.ticker = function(id)
 	if (pos >= slider.clientWidth)
 	    pos = slider.clientWidth - knob.clientWidth;;
 
-	knob.style.setProperty("left",
+	knob.style.setProperty(move,
 			       pos+"px",
 			       "important");
     }
